/*//////////////////////////////////////////////////////////////////////////////
Éléments JavaScript pour l'administration de Cahier de Prépa

* Les éléments de classe "editable" peuvent être transformés en mode édition par
la fonction transforme, automatiquement exécutée. L'attribut data-id est alors
nécessaire, et doit être de la forme table|champ|id.
* Les éléments de classe "edithtml" sont considérés comme contenant des
informations présentées en html : la fonction textareahtml crée alors un élément
de type textarea, un élément de type div éditable (contenteditable=true), et
ajoute des boutons facilitant les modifications. La textarea et la div éditable
sont alternativement visibles, à la demande.
* Les éléments possédant un attribut data-placeholder ont droit, par la fonction
placeholder, à une valeur affichée lorsque l'élément (en mode édition) est vide.
* Les liens de classe icon-aide lancent automatiquement une aide par la fonction
popup. Le contenu est récupéré dans les div de classe aide-xxx où xxx est la
valeur de l'attribut data-id du lien (ces div sont automatiquement non affichées
en css).
//////////////////////////////////////////////////////////////////////////////*/


///////////////
// Affichage //
///////////////

// Notification de résultat de requête AJAX
function affiche(message,etat) {
  $('#log').removeClass().addClass(etat).html(message).append('<span class="icon-ferme"></span>').show().off("click").on("click",function() {
    window.clearTimeout(t);
    $(this).hide(500);
  });
  var t = window.setTimeout(function() {
    $('#log').hide(500);
  },10000);
}

// Demande de reconnexion si timeout
function afficher_login(settings) {
  // Suppression d'une éventuelle fenêtre de la fonction popup existante
  $('#fenetre,#fenetre_fond').remove();
  // Création de la fenêtre d'indentification
  popup('<a class="icon-ok" title="Valider"></a><h3>Connexion nécessaire</h3>\
  <p>Vous avez été automatiquement déconnecté. Vous devez vous connecter à nouveau pour valider vos modifications.</p>\
  <form>\
  <p class="ligne"><label for="login">Identifiant&nbsp;: </label><input type="text" name="login" id="login"></p>\
  <p class="ligne"><label for="motdepasse">Mot de passe&nbsp;: </label><input type="password" name="motdepasse" id="motdepasse"></p>\
  </form>',true);
  $('#login').focus();
  // Envoi
  $('#fenetre a.icon-ok').on("click",function () {
    $.ajax({url: 'ajax.php',
            method: "post",
            data: $("#fenetre form").serialize()+'&'+settings.data,
            dataType: 'json',
            el: '',
            fonction: function() {
              return true;
            }
    }).success( function(data) {
      // Si erreur d'identification, on reste bloqué là
      if ( data['etat'] == 'login_nok' )
        $('#fenetre > p').html(data['message']).addClass('warning');
      // Sinon, et si l'identification a fonctionné, on exécute la fonction
      // passée dans l'argument settings, qui doit contenir la fonction fonction
      // et l'élément el. Remarque : les événements globaux ajaxSend, ajaxStop
      // et ajaxSuccess s'appliquent bien. 
      else {
        $('#fenetre,#fenetre_fond').remove();
        if ( data['etat'] == 'ok' )
          settings.fonction(settings.el);
      }
     });
  });
  // À la suppression : ajout d'une notification
  $('#fenetre a.icon-ferme').on("click",function () {
    affiche('Modification non effectuée, connexion nécessaire','nok');
  });
  // Envoi par appui sur Entrée si input
  $('#fenetre input').on('keypress',function (e) {
    if (e.which == 13)
      $('#fenetre a.icon-ok').click();
  });
}

// Affichage par-dessus le contenu de la page
// * contenu : chaine en HTML qui sera utilisée comme contenu de la fenêtre
// * modal : si true, fenêtre modale (impossible de continuer à éditer la page)
function popup(contenu,modal) {
  // Suppression d'une éventuelle fenêtre de la fonction popup existante
  $('#fenetre,#fenetre_fond').remove();
  // Création
  var el = $('<div id="fenetre"></div>').appendTo('body').html(contenu).focus();
  if ( modal )
    $('<div id="fenetre_fond"></div>').appendTo('body').click(function() {
      $('#fenetre,#fenetre_fond').remove();
    });
  // Si fenêtre non modale, possibilité de l'épingler en haut de la page
  else
    $('<a class="icon-epingle" title="Épingler à la page"></a>').prependTo(el).on("click",function() {
      $('#fenetre_fond').remove();
      $(this).remove();
      $('section').children('div,article').first().before(el.removeAttr('id'));
    });
  // Bouton de fermeture
  $('<a class="icon-ferme" title="Fermer"></a>').prependTo(el).on("click",function() {
    el.remove();
    $('#fenetre_fond').remove();
  });
}

//////////////////////////////////////////////////////////////
// Édition de textes : facilités de modication des éléments //
//////////////////////////////////////////////////////////////

// Affichage d'un placeholder (valeur si élement vide)
// Fonction à appliquer sur les input, textarea et div éditables, qui possèdent
// un attribut data-placeholder, éventuellement après textareahtml
$.fn.placeholder = function() {
  this.each( function() {
    var el = $(this);
    // Pour les div éditables, réalisation par css : ajout/suppression
    // automatique de la classe placeholder à chaque modification
    if ( el.is('div') )
      el.on("change keyup mouseup",function() {
        if ( $(this).text().length )
          $(this).removeClass('placeholder');
        else
          $(this).addClass('placeholder');
      }).change();
    // Pour les input et textarea, on crée un span avant l'élément, qui sera
    // automatiquement affiché ou caché à chaque modification de l'élément
    else {
      $('<span class="placeholder">'+this.getAttribute('data-placeholder')+'</span>').insertBefore(el).on('click',function() {
        $(this).next().focus();
      }).hide();
      el.on("change keyup mouseup",function() {
        if ( !(this.value.length) && ( $(this).is(':visible') ) )
          // bug avec show() : donne du display:block pour certains éléments
          $(this).prev().css('display','inline');
        else
          $(this).prev().hide();
      }).change();
    }
  });
}

// Transformation d'une textarea pour l'édition de code HTML
// Ajoute des boutons d'édition et la possibilité de commuter l'affichage avec
// une div éditable. Fonction à appliquer sur les textarea de classe edithtml
$.fn.textareahtml = function() {
  this.each(function() {
    var ta = $(this);
    var placeholder = this.getAttribute('data-placeholder');
    // Modification du placeholder
    this.setAttribute('data-placeholder',placeholder+'. Formattage en HTML, balises visibles.');
    // Ajout d'éléments : boutons, div éditable
    var ce = $('<div contenteditable="true" data-placeholder="'+placeholder+'"></div>').insertAfter(ta.before(boutons)).hide();
    var boutonretour = ta.prev().children(".icon-retour");
    // Retour à la ligne par Entrée, nettoyage au copié-collé
    ta.on("keypress",function(e) {
        if (e.which == 13)
          this.value = nettoie(this.value);
      })
      .on("paste cut",function() {
        var el = this;
        setTimeout(function() {
          el.value = nettoie(el.value);
        }, 100);
      });
    ce.on("keypress",function(e) {
        if (e.which == 13)
          boutonretour.click();
      })
      .on("paste cut",function() {
        var el = this;
        setTimeout(function() {
          el.innerHTML = nettoie(el.innerHTML)+'<br>';
        }, 100);
      });
    // Clic bouton "nosource" : passage de textarea à div
    ta.prev().children('.icon-nosource').on("click", function(e) {
      e.preventDefault();
      // Modification des visibilités
      ta.hide();
      ta.prev().hide();
      ce.show().css("min-height",ta.outerHeight());
      $(this).hide().prev().show();
      // Nettoyage et synchronisation (change -> mise à jour du placeholder)
      ce.focus().html(nettoie(ta.val())).change();
      // Mise en place du curseur à la fin
      if ( window.getSelection ) {
        var r = document.createRange();
        r.selectNodeContents(ce[0]);
        r.collapse(false);
        var s = window.getSelection();
        s.removeAllRanges();
        s.addRange(r);
      }
      else {
        var r = document.body.createTextRange();
        r.moveToElementText(ce[0]);
        r.collapse(false);
        r.select();
      }
    });
    // Clic bouton "source" : passage de div à textarea
    ta.prev().children('.icon-source').on("click", function(e) {
      e.preventDefault();
      // Modification des visibilités
      ce.hide(0);
      ta.show(0).css("height",ce.height());
      $(this).hide().next().show();
      // Nettoyage et synchronisation (change -> mise à jour du placeholder)
      ta.focus().val(nettoie(ce.html())).change();
    }).hide();
    // Clic bouton aide
    ta.prev().children('.icon-aide').on("click",function(e) {
      e.preventDefault();
      aidetexte();
    });
    // Autres clics
    ta.prev().children().not('.icon-nosource,.icon-source,.icon-aide').on("click",function(e) {
      e.preventDefault();
      window['insertion_'+this.className.substring(5)]($(this));
    });
  });
}

// Édition "en place"
// Fonction à appliquer sur les éléments de classe editable
$.fn.editinplace = function() {
  this.each(function() {
    var el = $(this);
    // Enregistrement de la valeur originale, pour rétablissement si abandon
    this.setAttribute('data-original',( el.is('h3') ) ? el.text() : el.html());
    // Transformation
    $('<a class="icon-edite" title="Modifier"></a>').appendTo(el).on("click",transforme);
  });
}

// Transformation d'un élément h3/div de classe editable en input/textarea
function transforme() {
  var el = $(this).parent().addClass('avecform');
  // Création d'une textarea si div, d'un input sinon
  if ( el.is('div') )
    el.html('<form><textarea name="val" rows="'+(el.attr('data-original').split(/\r\n|\r|\n/).length+3)+'"></textarea></form>');
  else
    el.html('<form class="edition" onsubmit="$(this).children(\'a.icon-ok\').click(); return false;"><input type="text" name="val" value=""></form>');
  // Identifiant de l'input ou de la textarea
  var input = el.find('[name="val"]').val(el.attr('data-original'));
  // Facilités (boutons si edithtml, placeholder)
  if ( el.attr('data-placeholder') ) {
    input.attr('data-placeholder',el.attr('data-placeholder'));
    if ( el.hasClass('edithtml') ) {
      input.textareahtml(); input.next().placeholder();
    }
    input.placeholder();
  }
  // Envoi
  $('<a class="icon-ok" title="Valider"></a>').appendTo(el.children()).on("click",function() {
    // Identifiant de l'entrée à modifier : table|champ|id
    var id = el.attr('data-id').split('|');
    // Nettoyage et synchronisation si besoin
    if (el.hasClass('edithtml') )
      input.val(nettoie( ( input.is(':visible') ) ? input.val() : input.next().html() ));
    // Envoi
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { table:id[0], champ:id[1], id:id[2], val:input.val() },
            dataType: 'json',
            el: el,
            fonction: function(el) {
              var val = el.find('[name="val"]').val();
              el.removeClass('avecform').html(val).attr('data-original',val);
              $('<a class="icon-edite" title="Modifier"></a>').appendTo(el).on("click",transforme);
            }
    });
  });
  // Annulation
  $('<a class="icon-annule" title="Annuler"></a>').appendTo(el.children()).on("click",function() {
    el.removeClass('avecform').html(el.attr('data-original'));
    $('<a class="icon-edite" title="Modifier"></a>').appendTo(el).on("click",transforme);
  });
  // Récupération du focus avec curseur à la fin
  input.focus().val( ( el.hasClass('edithtml') ) ? nettoie(input.val()) : input.val() );
}

// Édition "en place" des propriétés des éléments des cahiers de texte
// Fonction à appliquer sur les éléments de classe titrecdt
$.fn.editinplacecdt = function() {
  this.each(function() {
    // Enregistrement de la valeur originale, pour rétablissement si abandon
    $(this).wrapInner('<span></span>').attr('data-original',$(this).text());
    // Transformation
    $('<a class="icon-edite" title="Modifier"></a>').appendTo($(this)).on("click",transformecdt);
  });
}

// Transformation d'un élément de classe titrecdt pour son édition
function transformecdt() {
  var el = $(this).parent();
  el.children('.icon-edite').remove();
  var form = $('<form class="titrecdt"></form>').insertBefore(el.parent().children('div')).html($('#form-cdt').html());
  // Mise en place des facilités  
  form.boutonscdt();
  // Mise à jour du titre si modification
  form.find('input,[name="demigroupe"]').on('change keyup', function() {
    var t = new Date(form.find('[name="jour"]').val().replace(/(.{2})\/(.{2})\/(.{4})/,function(tout,x,y,z) {return z+'-'+y+'-'+x; }));
    var dg = ( form.find('[name="demigroupe"]').val() == 1 ) ? ' (en demi-groupe)' : '';
    switch ( parseInt(seances[form.find('[name="tid"]').val()]) ) {
      case 0:
        var titre = jours[t.getDay()]+' '+form.find('[name="jour"]').val()+' à '+form.find('[name="h_debut"]').val()+' : '+form.find('[name="tid"] option:selected').text()+dg;
        break;
      case 1:
        var titre = jours[t.getDay()]+' '+form.find('[name="jour"]').val()+' de '+form.find('[name="h_debut"]').val()+' à '+form.find('[name="h_fin"]').val()+' : '+form.find('[name="tid"] option:selected').text()+dg;
        break;
      case 2:
        var titre = jours[t.getDay()]+' '+form.find('[name="jour"]').val()+' : '+form.find('[name="tid"] option:selected').text()+' pour le '+form.find('[name="pour"]').val()+dg;
        break;
      case 3:
        var titre = jours[t.getDay()]+' '+form.find('[name="jour"]').val()+' : '+form.find('[name="tid"] option:selected').text()+dg;
        break;
      case 4:
        var titre = jours[t.getDay()]+' '+form.find('[name="jour"]').val();
        break;
      case 5:
        var titre = '[Entrée hebdomadaire]';
    }
    el.children('span').text(titre);
  });
  // Récupération des valeurs et modification initiale du formulaire
  var valeurs = $.parseJSON(el.attr('data-donnees'));
  for ( var cle in valeurs )
    form.find('[name="'+cle+'"]').val(valeurs[cle]);
  // Envoi
  $('<a class="icon-ok" title="Valider"></a>').appendTo(el).on("click",function() {
    // Identifiant de l'entrée à modifier : table|id
    var id = el.parent().attr('data-id').split('|');
    // Envoi
    $.ajax({url: 'ajax.php',
            method: "post",
            data: 'table=cdt&id='+id[1]+'&'+form.serialize(),
            dataType: 'json',
            el: el,
            fonction: function(el) {
              var form = el.siblings('form');
              el.attr('data-original',el.children('span').text()).attr('data-donnees','{"tid":'+form.find('[name="tid"]').val()+',"jour":"'+form.find('[name="jour"]').val()+'","h_debut":"'+form.find('[name="h_debut"]').val()+'","h_fin":"'+form.find('[name="h_fin"]').val()+'","pour":"'+form.find('[name="pour"]').val()+'","demigroupe":'+form.find('[name="demigroupe"]').val()+'}');
              form.remove();
              el.children('a').remove();
              $('<a class="icon-edite" title="Modifier"></a>').appendTo(el).on("click",transformecdt);
            }
    }).success( function(data) {
      if ( ( data['etat'] == 'ok' ) && ( data['reload'] == 'oui' ) )
        location.reload(true);
    });
  });
  // Annulation
  $('<a class="icon-annule" title="Annuler"></a>').appendTo(el).on("click",function() {
    form.remove();
    el.children('span').html(el.attr('data-original'));
    el.children('a').remove();
    $('<a class="icon-edite" title="Modifier"></a>').appendTo(el).on("click",transformecdt);
  });
}

// Facilités du formulaire de modification des propriétés des éléments de cahier
// de texte, utilisé par transformecdt() (élément existant) et formulaire()
// (nouvel élément)
$.fn.boutonscdt = function() {
  form = this;
  form.find('.date').datepick({ dateFormat: 'dd/mm/yyyy', showTrigger: '<img src="js/calendar-blue.gif">', onSelect: function() { form.find('[name="h_debut"]').change(); } });
  form.find('.heure').timeEntry({timeSteps: [1, 15, 0], separator: 'h', spinnerImage: 'js/spinnerDefault.png'});
  // Ajout du zéro devant les dates et heures à 1 chiffre
  var zero = function(n) {
    return ( String(n).length == 1 ) ? '0'+n : String(n);
  }
  // Action lancée à la modification du raccourci
  form.find('[name="raccourci"]').on('change keyup', function() {
    var valeurs = raccourcis[this.value];
    for ( var cle in valeurs ) {
      // Modification de la date (prochain lundi/mardi..)
      if ( cle == 'jour' ) {
        var t = new Date;
        var j = parseInt(valeurs['jour']);
        t.setDate( ( j > t.getDay() ) ? t.getDate()-t.getDay()-7+j : t.getDate()-t.getDay()+j );
        form.find('[name="jour"]').val(zero(t.getDate())+'/'+zero(t.getMonth()+1)+'/'+t.getFullYear());
      }
      // Modification des autres jours
      else
        form.find('[name="'+cle+'"]').val(valeurs[cle]);
    }
    // Pour éviter d'être remis à zéro immédiatement
    this.setAttribute('data-modif',1);
    // Modifie les champs visibles
    form.find('[name="tid"]').change();
  }).attr('data-modif',0);
  // Action lancée à la modification du type de séance
  form.find('[name="tid"]').on('change keyup', function() {
    switch ( parseInt(seances[this.value]) ) {
      case 0:
        form.find('[name="h_debut"]').parent().show();
        form.find('[name="h_fin"]').parent().hide();
        form.find('[name="pour"]').parent().hide();
        form.find('[name="demigroupe"]').parent().show();
        break;
      case 1:
        form.find('[name="h_debut"]').parent().show();
        form.find('[name="h_fin"]').parent().show();
        form.find('[name="pour"]').parent().hide();
        form.find('[name="demigroupe"]').parent().show();
        break;
      case 2:
        form.find('[name="h_debut"]').parent().hide();
        form.find('[name="h_fin"]').parent().hide();
        form.find('[name="pour"]').parent().show();
        form.find('[name="demigroupe"]').parent().show();
        break;
      case 3:
        form.find('[name="h_debut"]').parent().hide();
        form.find('[name="h_fin"]').parent().hide();
        form.find('[name="pour"]').parent().hide();
        form.find('[name="demigroupe"]').parent().show();
        break;
      default:
        form.find('[name="h_debut"]').parent().hide();
        form.find('[name="h_fin"]').parent().hide();
        form.find('[name="pour"]').parent().hide();
        form.find('[name="demigroupe"]').parent().hide();
        break;
    }
    // Mise à jour du champ de raccourci
    form.find('[name="jour"]').change();
  });
  // Action lancée à la modification des autres champs
  form.find('input,[name="demigroupe"]').on('change keyup', function() {
    // Remise à zéro du raccourci s'il n'a pas été modifié immédiatement
    if ( form.find('[name="raccourci"]').attr('data-modif') == 0 )
      form.find('[name="raccourci"]').val(0);
    else
      form.find('[name="raccourci"]').attr('data-modif',0);
  });
  // Envoi par appui sur Entrée
  form.find('input,select').on("keypress",function(e) {
    if ( e.which == 13 )
      el.find('a.icon-ok').click();
  });
  // Focus sur le premier champ et modification initiale
  form.find('select:first').focus();
  form.find('[name="tid"]').change();
}

// Facilités du formulaire de modification des propriétés des raccourcis de
// cahier de texte, utilisé sur les éléments de classe cdt-raccourcis
$.fn.affichechamps = function() {
  this.each(function() {
    var form = $(this);
    form.find('.heure').timeEntry({timeSteps: [1, 15, 0], separator: 'h', spinnerImage: 'js/spinnerDefault.png'});
    // Action lancée à la modification du type de séance
    form.find('[name="type"]').on('change keyup', function() {
      switch ( parseInt(seances[this.value]) ) {
        case 0:
          form.find('[name="h_debut"]').parent().show();
          form.find('[name="h_fin"]').parent().hide();
          form.find('[name="demigroupe"]').parent().show();
          break;
        case 1:
          form.find('[name="h_debut"]').parent().show();
          form.find('[name="h_fin"]').parent().show();
          form.find('[name="demigroupe"]').parent().show();
          break;
        case 2:
          form.find('[name="h_debut"]').parent().hide();
          form.find('[name="h_fin"]').parent().hide();
          form.find('[name="demigroupe"]').parent().show();
          break;
        case 3:
          form.find('[name="h_debut"]').parent().hide();
          form.find('[name="h_fin"]').parent().hide();
          form.find('[name="demigroupe"]').parent().show();
          break;
        default:
          form.find('[name="h_debut"]').parent().hide();
          form.find('[name="h_fin"]').parent().hide();
          form.find('[name="demigroupe"]').parent().hide();
          break;
      }
    }).change();
    // Envoi par appui sur Entrée
    form.find('input,select').on("keypress",function(e) {
      if ( e.which == 13 )
        form.find('a.icon-ok').click();
    });
  });
}

// Nettoyage des contenus de textarea pour l'édition de code HTML
function nettoie(html) {
  // Suppression du span cdptmp ajouté par la fonction insert()
  if ( html.indexOf('cdptmp') > 0 ) {
    // Suppression des spans non vides
    var tmp = $('<div>'+html+'</div>');
    tmp.find('.cdptmp').contents().unwrap();
    html = tmp.html();
    // Suppression des spans restants, vides
    if ( html.indexOf('cdptmp') > 0 )
      html = html.replace(/<span class="cdptmp"><\/span>/g,'');
  }
  // Autres modifications
  return html.replace(/(<\/?[A-Z]+)([^>]*>)/g, function (tout,x,y) { return x.toLowerCase()+y; })  // Minuscules pour les balises
             .replace(/[\r\n ]+/g,' ')  // Suppression des retours à la ligne et espaces multiples
             .replace(/(<br>)+[ ]?<\/(p|div|li|h)/g, function (tout,x,y) { return '</'+y; }).replace(/<br>/g, '<br>\n')  // Suppression des <br> multiples
             .replace(/<(p|div|li|h)/g, function (x) { return '\n'+x; })  // Retour à la ligne avant <p>, <div>, <li>, <h*>
             .replace(/<\/(p|div|li|h.)>/g, function (x) { return x+'\n'; }) // Retour à la ligne après </p>, </div>, </li>, </h*>
             .replace(/<\/?(ul|ol)[^>]*>/g, function (x) { return '\n'+x+'\n'; })  // Retour à la ligne avant et après <ul>, </ul>, <ol>, </ol>
             // Formattage en paragraphe d'une ligne finie par <br> et non commencée par <p>, <div>, <ul>, <ol>, <li>, <h*>
             .replace(/^(?!(<p|<div|<ul|<ol|<li|<h))(.+)<br>$/gm, function (tout,x,y) { return '<p>'+y+'</p>'; })
             // Formattage en paragraphe d'une ligne non commencée par <p>, <div>, <ul>, <ol>, <li>, <h*>, non fermée par </p>, </div>, </ul>, </ol>, </li>, </h*>
             .replace(/^(?!(<(p|div|ul|ol|li)))[ ]?(.+)[ ]?$/gm, function (t,x,y,z) { return ( z.match(/.*(p|div|ul|ol|li|h.)>$/) ) ? z : '<p>'+z+'</p>'; })
             // Suppression des lignes contenant une seule balise <p>, </p>, <div>, </div>, <h*>, </h*>, <br> (avec éventuellement des espaces autour)
             // Suppression des lignes de paragraphes/div/titres vides (contenant des espaces ou un <br>)
             .replace(/^[ ]?(<\/?(br|p|div|h.)>){0,2}[ ]?(<\/(p|div|h.)>)?[ ]?$/gm,'').replace(/^\n/gm,'')
             // Indentation devant <li>
             .replace(/<li/g,'  <li');
}

// Insertion lancée par les boutons.
// Fonction générique pour l'utilisation des boutons insérés pour les textarea
// d'édition en HTML. Arguments :
//  * el = identifiant jquery du bouton
//  * debut = ce qui sera inséré avant la sélection
//  * fin = ce qui sera inséré après la sélection
//  * milieu = ce qui sera inséré à la place de la sélection (si renseigné)
// Après insertion, la sélection est conservée (ou placée sur milieu)
function insert(el,debut,fin,milieu) {
  // Récupération et modification du contenu
  var contenant = el.parent().siblings('textarea,[contenteditable]').filter(':visible')[0];
  if ( !contenant.hasAttribute('data-selection') )
    marqueselection(el);
  var texte = ( milieu === undefined ) ? debut+'Í'+contenant.getAttribute('data-selection')+'Ì'+fin : debut+'Í'+milieu+'Ì'+fin;
  var contenu = nettoie(contenant.getAttribute('data-contenu').replace(/Í.*Ì/,texte));
  // Affichage
  if ( contenant.tagName == 'TEXTAREA' )
    contenant.value = contenu.replace(/[ÍÌ]/g,'');
  else
    contenant.innerHTML = contenu.replace(/[ÍÌ]/g,'');
  // Suppression des attributs liés à la sélection
  marqueselection(el,true);
  // Mise à jour éventuelle du placeholder
  $(contenant).change();
  // Resélection
  // Cas des textarea, navigateurs modernes
  if ( ( contenant.tagName == 'TEXTAREA' ) && ( contenant.selectionStart !== undefined ) ) {
    contenant.selectionStart = contenu.indexOf('Í');
    contenant.selectionEnd = contenu.indexOf('Ì')-1;
    contenant.focus();
  }
  // Cas des textarea et divs éditables, navigateurs anciens (IE<9)
  else if ( document.selection ) {
    // On doit compter les caractères de texte uniquement : suppression des
    // balises si contenant est div editable
    if ( contenant.tagName != 'TEXTAREA' )
      contenu = contenu.replace(/(<([^>]+)>)[\n]*/g,'');
    range = document.body.createTextRange();
    range.moveToElementText(contenant);
    range.collapse(true); 
    range.moveEnd("character", contenu.indexOf('Ì')-1);
    range.moveStart("character", contenu.indexOf('Í'));
    range.select();
  }
  // Cas des divs éditables, navigateurs modernes
  else if (window.getSelection) {
    contenant.innerHTML = contenu.replace('Í','<span class="cdptmp">').replace('Ì','</span>')+'<br>';
    selection = window.getSelection();
    range = document.createRange();
    range.selectNodeContents($(contenant).find('.cdptmp')[0]);
    selection.removeAllRanges();
    selection.addRange(range);
    contenant.focus();
  }
}

// Marquage de la sélection
// Crée deux attributs sur la textarea/div éditable visible :
//  * data-selection = la sélection en cours
//  * data-contenu = le contenu entier de la textarea/div éditable, où la
// sélection est encadrée par Í (début) et Ì (fin)
// Retourne le texte effectivement sélectionné. Arguments : 
//  * el = identifiant jquery du bouton "appelant" (via la fonction insert)
//  * efface = si true, on efface simplement les attributs
function marqueselection(el,efface) {
  var contenant = el.parent().siblings('textarea,[contenteditable]').filter(':visible')[0];
  if ( efface ) {
    contenant.removeAttribute('data-selection');
    contenant.removeAttribute('data-contenu');
    return true;
  }
  var original = ( contenant.tagName == 'TEXTAREA' ) ? contenant.value : contenant.innerHTML;
  var sel = '';
  // Cas des textarea, navigateurs modernes
  if ( ( contenant.tagName == 'TEXTAREA' ) && ( contenant.selectionStart !== undefined ) ) {
    contenant.focus();
    sel = contenant.value.substring(contenant.selectionStart,contenant.selectionEnd);
    contenant.value = contenant.value.substr(0,contenant.selectionStart)+'Í'+sel+'Ì'+contenant.value.substring(contenant.selectionEnd);
  }
  // Cas des divs éditables, navigateurs modernes
  else if ( window.getSelection ) {
    var range = window.getSelection().getRangeAt(0);
    if ( ( contenant == range.commonAncestorContainer ) || $.contains(contenant,range.commonAncestorContainer) ) {
      var sel = window.getSelection().toString();
      range.deleteContents();
      range.insertNode(document.createTextNode('Í'+sel+'Ì'));
    }
  }
  // Cas des navigateurs anciens (IE<9)
  else {
    var range = document.selection.createRange();
    if ( ( contenant == range.parentElement() ) || $.contains(contenant,range.parentElement()) ) {
      var sel = document.selection.createRange().text;
      document.selection.createRange().text = 'Í'+sel+'Ì';
    }
  }
  // Remise à l'orgine
  if ( contenant.tagName == 'TEXTAREA' ) {
    var contenu = contenant.value;
    contenant.value = original;
  }
  else {
    var contenu = contenant.innerHTML;
    $(contenant).html(original); // Bug IE8 : modification de innerHTML impossible
  }
  // Par défaut : sélection à la fin
  if ( contenu.indexOf('Ì') < 0 )
    contenu = contenu + 'ÍÌ';
  // Enregistrement et retour
  contenant.setAttribute('data-selection',sel);
  contenant.setAttribute('data-contenu',contenu);
  return sel;
}

////////////////////////////////////////////////////////////
// Édition de textes : boutons pour les textarea.edithtml //
////////////////////////////////////////////////////////////

// Définition des boutons
var boutons = '\
<p class="boutons">\
  <button class="icon-titres" title="Niveaux de titres"></button>\
  <button class="icon-par1" title="Paragraphe"></button>\
  <button class="icon-par2" title="Paragraphe important"></button>\
  <button class="icon-par3" title="Paragraphe très important"></button>\
  <button class="icon-retour" title="Retour à la ligne"></button>\
  <button class="icon-gras" title="Gras"></button>\
  <button class="icon-italique" title="Italique"></button>\
  <button class="icon-souligne" title="Souligné"></button>\
  <button class="icon-omega" title="Insérer une lettre grecque"></button>\
  <button class="icon-sigma" title="Insérer un signe mathématique"></button>\
  <button class="icon-exp" title="Exposant"></button>\
  <button class="icon-ind" title="Indice"></button>\
  <button class="icon-ol" title="Liste énumérée"></button>\
  <button class="icon-ul" title="Liste à puces"></button>\
  <button class="icon-lien1" title="Lien vers un document du site"></button>\
  <button class="icon-lien2" title="Lien internet"></button>\
  <button class="icon-tex" title="LATEX!"></button>\
  <button class="icon-source" title="Voir et éditer le code html"></button>\
  <button class="icon-nosource" title="Voir et éditer le texte formaté"></button>\
  <button class="icon-aide" title="Voir et éditer le texte formaté"></button>\
</p>';

// Fonctions lancées par les boutons appelant une fenêtre par la fonction popup
function insertion_titres(el) {
  popup('<a class="icon-ok" title="Valider"></a><h3>Insertion d\'un titre</h3>\
  <p>Choisissez le type du titre ci-dessous. Vous pouvez éventuellement modifier le texte (ou pourrez le faire ultérieurement). Il est conseillé d\'utiliser des titres de niveau 2 pour les titres dans les programmes de colle.</p>\
  <input type="radio" name="titre" id="t3" value="3" checked><h3><label for="t3">Titre de niveau 1 (pour les I,II...)</label></h3><br>\
  <input type="radio" name="titre" id="t4" value="4"><h4><label for="t4">Titre de niveau 2 (pour les 1,2...)</label></h4><br>\
  <input type="radio" name="titre" id="t5" value="5"><h5><label for="t5">Titre de niveau 3 (pour les a,b...)</label></h5><br>\
  <input type="radio" name="titre" id="t6" value="6"><h6><label for="t6">Titre de niveau 4</label></h6><br>\
  <p class="ligne"><label for="texte">Texte&nbsp;: </label><input type="text" id="texte" value="'+marqueselection(el)+'" size="80"></p>\
  <hr><h3>Aperçu</h3><div id="apercu"></div>',true);
  // Modification automatique de l'aperçu
  $('#fenetre input').on("click keyup", function () {
    var balise = 'h'+$("[name='titre']:checked").val();
    $('#apercu').html('<'+balise+'>'+( ( $('#texte').val().length ) ? $('#texte').val() : 'Texte du titre' )+'</'+balise+'>');
  }).first().keyup();
  // Insertion par appui sur Entrée
  $('#texte').on("keypress",function(e) {
    if ( e.which == 13 )
      $('#fenetre a.icon-ok').click();
  }).focus();
  // Insertion
  $('#fenetre a.icon-ok').on("click",function () {
    var balise = 'h'+$("[name='titre']:checked").val();
    insert(el,'<'+balise+'>','</'+balise+'>',$('#texte').val());
    $('#fenetre,#fenetre_fond').remove();
  });
  // Suppression des attributs liés à la sélection lors de la fermeture
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
}

function insertion_omega(el) {
  popup('<h3>Insertion d\'une lettre grecque</h3>\
  <p>Cliquez sur la lettre à insérer&nbsp;:</p>\
  <button>&alpha;</button> <button>&beta;</button> <button>&gamma;</button> <button>&Delta;</button> <button>&delta;</button> <button>&epsilon;</button> <button>&eta;</button> <button>&Theta;</button> <button>&theta;</button> <button>&Lambda;</button> <button>&lambda;</button> <button>&mu;</button> <button>&nu;</button> <button>&xi;</button> <button>&Pi;</button> <button>&pi;</button> <button>&rho;</button> <button>&Sigma;</button> <button>&sigma;</button> <button>&tau;</button> <button>&upsilon;</button> <button>&Phi;</button> <button>&phi;</button> <button>&Psi;</button> <button>&psi;</button> <button>&Omega;</button> <button>&omega;</button>',true);
  $('#fenetre button').on("click",function () {
    insert(el,'','',$(this).text());
    $('#fenetre,#fenetre_fond').remove();
  });
}

function insertion_sigma(el) {
  popup('<h3>Insertion d\'un symbole mathématique</h3>\
  <p>Cliquez sur le symbole à insérer&nbsp;:</p>\
  <button>&forall;</button> <button>&exist;</button> <button>&part;</button> <button>&nabla;</button> <button>&prod;</button> <button>&sum;</button> <button>&plusmn;</button> <button>&radic;</button> <button>&infin;</button> <button>&int;</button> <button>&prop;</button> <button>&sim;</button> <button>&cong;</button> <button>&asymp;</button> <button>&ne;</button> <button>&equiv;</button> <button>&le;</button> <button>&ge;</button> <button>&sub;</button> <button>&sup;</button> <button>&nsub;</button> <button>&sube;</button> <button>&supe;</button> <button>&isin;</button> <button>&notin;</button> <button>&ni;</button> <button>&oplus;</button> <button>&otimes;</button> <button>&sdot;</button> <button>&and;</button> <button>&or;</button> <button>&cap;</button> <button>&cup;</button> <button>&real;</button> <button>&image;</button> <button>&empty;</button> <button>&deg;</button> <button>&prime;</button> <button>&micro;</button> <button>&larr;</button> <button>&uarr;</button> <button>&rarr;</button> <button>&darr;</button> <button>&harr;</button> <button>&lArr;</button> <button>&uArr;</button> <button>&rArr;</button> <button>&dArr;</button> <button>&hArr;</button>',true);
  $('#fenetre button').on("click",function () {
    insert(el,'','',$(this).text());
    $('#fenetre,#fenetre_fond').remove();
  });
}

function insertion_ol(el) {
  popup('<a class="icon-ok" title="Valider"></a><h3>Insertion d\'une liste numérotée</h3>\
  <p>Choisissez le type de numérotation et la valeur de départ de la liste ci-dessous. Vous pouvez éventuellement modifier les différents éléments en les écrivant ligne par ligne. Vous pourrez ajouter un élément ultérieurement en l\'encadrant par les balises &lt;li&gt; et &lt;/li&gt;.</p>\
  <p class="ligne"><label for="t1">Numérotation numérique (1, 2, 3...)</label><input type="radio" name="type" id="t1" value="1" checked></p>\
  <p class="ligne"><label for="t2">Numérotation alphabétique majuscule (A, B, C...)</label><input type="radio" name="type" id="t2" value="A"></p>\
  <p class="ligne"><label for="t3">Numérotation alphabétique minuscule (a, b, c...)</label><input type="radio" name="type" id="t3" value="a"></p>\
  <p class="ligne"><label for="t4">Numérotation romaine majuscule (I, II, III...)</label><input type="radio" name="type" id="t4" value="I"></p>\
  <p class="ligne"><label for="t5">Numérotation romaine minuscule (i, ii, iii...)</label><input type="radio" name="type" id="t5" value="i"></p>\
  <p class="ligne"><label for="debut">Valeur de début (numérique)</label><input type="text" id="debut" value="1"></p>\
  <p class="ligne"><label for="lignes">Textes (chaque ligne correspond à un élément de la liste)&nbsp;: </label></p>\
  <textarea id="lignes" rows="5">'+marqueselection(el)+'</textarea>\
  <hr><h3>Aperçu</h3><div id="apercu"></div>',true);
  // Modification automatique de l'aperçu
  $('#fenetre :input').on("click keyup", function () {
    var debut = $('#debut').val();
    debut = ( debut.length && ( debut > 1 ) ) ? ' start="'+debut+'"' : '';
    $('#apercu').html('<ol type="'+$("[name='type']:checked").val()+'"'+debut+'><li>'
                       +( ( $('#lignes').val().length ) ? $('#lignes').val().trim('\n').replace(/\n/g,'</li><li>') : 'Première ligne</li><li>Deuxième ligne</li><li>...' )
                       +'</li></ol>');
  }).first().keyup();
  $('#lignes').focus();
  // Insertion
  $('#fenetre a.icon-ok').on("click",function () {
    var debut = $('#debut').val();
    debut = ( debut.length && ( debut > 1 ) ) ? ' start="'+debut+'"' : '';
    var elements = $('#lignes').val().trim('\n');
    // On ne souhaite garder en sélection que la dernière ligne
    var index = elements.lastIndexOf('\n');
    if ( index > 0 ) {
      var dernier = elements.substring(index+1);
      elements = elements.substring(0,index);
    }
    else
      var dernier = '';
    // Insertion
    insert(el,'<ol type="'+$("[name='type']:checked").val()+'"'+debut+'><li>'+elements.replace(/\n/g,'</li><li>')+'</li><li>','</li></ol>',dernier);
    $('#fenetre,#fenetre_fond').remove();
  });
  // Suppression des attributs liés à la sélection lors de la fermeture
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
}

function insertion_ul(el) {
  popup('<a class="icon-ok" title="Valider"></a><h3>Insertion d\'une liste à puces</h3>\
  <p>Vous pouvez éventuellement modifier les différents éléments en les écrivant ligne par ligne (chaque ligne correspond à un élément de la la liste). Vous pourrez ajouter un élément ultérieurement en l\'encadrant par les balises &lt;li&gt; et &lt;/li&gt;.</p>\
  <textarea id="lignes" rows="5">'+marqueselection(el)+'</textarea>\
  <hr><h3>Aperçu</h3><div id="apercu"></div>',true);
  // Modification automatique de l'aperçu
  $('#lignes').on("click keyup", function () {
    $('#apercu').html('<ul><li>'
                      +( ( $('#lignes').val().length ) ? $('#lignes').val().trim('\n').replace(/\n/g,'</li><li>') : 'Première ligne</li><li>Deuxième ligne</li><li>...' )
                      +'</li></ul>');
  }).keyup().focus();
  // Insertion
  $('#fenetre a.icon-ok').on("click",function () {
    var elements = $('#lignes').val().trim('\n');
    // On ne souhaite garder en sélection que la dernière ligne
    var index = elements.lastIndexOf('\n');
    if ( index > 0 ) { var dernier = elements.substring(index+1); elements = elements.substring(0,index); }
    else var dernier = '';
    // Insertion
    insert(el,'<ul><li>'+elements.replace(/\n/g,'</li><li>')+'</li><li>','</li></ul>',dernier);
    $('#fenetre,#fenetre_fond').remove();
  });
  // Suppression des attributs liés à la sélection lors de la fermeture
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
}

function insertion_lien1(el) {
  var sel = marqueselection(el);
  // Préparation : fenêtre et fermeture
  popup('<a class="icon-ok" title="Valider"></a><h3>Insertion d\'un lien vers un document de Cahier de Prépa</h3>\
  <div><p style="text-align:center; margin: 2em 0;">[Récupération des listes de documents]</p></div>\
  <div style="display:none;"><hr><h3>Aperçu</h3><div id="apercu" style="text-align:center;">[Veuillez choisir un document]</div></div>',true);
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
  // Récupération des listes de répertoires et de documents
  $.ajax({url: 'ajax.php',
          method: "post",
          data: { recupdoc:'' },
          dataType: 'json'})
    .success( function(data) {
      // Fonction de mise à jour de l'aperçu
      var majapercu = function() {
        var apercu = $('#apercu');
        var id = $('#doc').val();
        var texte = $('#doc option:selected').text();
        // Rien à afficher : texte par défaut
        if ( id == 0 )
          apercu.html(texte);
        // Cas des pdfs affichés
        else if ( $('#vue').is(':checked') ) {
          var l = $('#largeur').val();
          if ( texte.slice(-4,-1) == 'pdf') {
            // Pas de pdf affiché précédemment
            if ( apercu.children('.pdf').length == 0 )
              apercu.html('<div><object data="download?id='+id+'" type="application/pdf" height="100%" width="100%"> <a href="download?id='+id+'">'+texte+'</a> </object></div>');
            // Changement de document
            else if ( apercu.find('object').attr('data').substr(12) != id )
              apercu.find('object').attr('data','download?id='+id).html('<a href="download?id='+id+'">'+texte+'</a>');
            // Changement de format
            apercu.children().attr('class','pdf '+$('#format').val());
            // Changement de largeur : seulement si largeur spécifiée
            if ( l ) {
              if ( l == 100 )
                apercu.children().removeAttr('style').children().attr('width','100%').removeAttr('style');
              else  {
                apercu.children().css('padding-bottom',($('<div class="'+$('#format').val()+'"></div>').css('padding-bottom').slice(0,-1)*l/100)+'%');
                apercu.find('object').attr('width',l+'%').css('left',(100-l)/2+'%');
              }
            }
          }
          // Cas des images affichées
          else if ( 'jpgpegpng'.indexOf(texte.slice(-4,-1)) > -1 ) {
            // Pas d'image affichée précédemment
            if ( apercu.children('img').length == 0 )
              apercu.css('text-align','').html('<img src="download?id='+id+'">');
            // Changement de document
            else if ( apercu.children().attr('src').substr(12) != id )
              apercu.children().attr('src','download?id='+id);
            // Changement de largeur : seulement si largeur spécifiée
            if ( l ) {
              if ( l == 100 )
                apercu.children().removeAttr('style');
              else
                apercu.children().css('width',l+'%').css('margin-left',(100-l)/2+'%');
            }
          }
        }
        // Cas des liens classiques
        else
          $('#apercu').css('text-align','center').html('<a onclick="return false;" href="download?id='+this.value+'">'+$('#texte').val()+'</a>');
      }

      // Fonction d'affichage
      var affichedocs = function(data) {
        $('#fenetre > div:first').html('\
  <p>Choisissez ci-dessous le répertoire puis le document à insérer. Vous pouvez aussi modifier le texte visible. Cela reste modifiable ultérieurement&nbsp;: le texte est situé entre les deux balises &lt;a...&gt; et &lt;/a&gt;.</p>\
  <p class="ligne"><label for="mat">Matière&nbsp;:</label><select id="mat">'+data.mats+'</select></p>\
  <p class="ligne"><label for="rep">Répertoire&nbsp;:</label><select id="rep"></select></p>\
  <p class="ligne"><label for="doc">Document&nbsp;:</label><select id="doc"></select></p>\
  <p class="ligne"><label for="texte">Texte visible&nbsp;:</label><input type="text" id="texte" value="'+sel+'" size="80" data-auto="1"></p>\
  <p class="ligne"><label for="vue">Afficher dans la page (PDF et image uniquement)</label><input type="checkbox" id="vue">\
  <p class="ligne"><label for="largeur">Largeur en %&nbsp;:</label><input type="text" id="largeur" value="100" size="3"></p>\
  <p class="ligne"><label for="format">Format (PDF uniquement)</label><select id="format">\
    <option value="portrait">A4 vertical</option><option value="paysage">A4 horizontal</option><option value="hauteur50">Hauteur 50%</option>\
  </select>');
        $('#fenetre > div:last').show();
        // L'attribut data-auto vaut 1 si la valeur du texte est automatiquement
        // modifiée, pour valoir toujours le nom du document. Il est positionné
        // à 0 dès que l'on modifie manuellement l'entrée #texte, redevient égal
        // à 1 si on la vide.
        if ( $('#texte').val().length )
          $('#texte').attr('data-auto',0);
        // Actions sur #doc
        $('#doc').on("change keyup", function (e) {
          if ( e.which == 13 )
            $('#fenetre a.icon-ok').click();
          var texte = $('#doc option:selected').text();
          // Mise à jour automatique du texte si data-auto vaut 1
          if ( $('#texte').attr('data-auto') == 1 )
            $('#texte').val( ( this.value > 0 ) ? texte.substr(0,texte.lastIndexOf('(')-1) : '---' );
          // Visibilité des cases à cocher
          if ( 'pdfjpgpegpng'.indexOf(texte.slice(-4,-1)) > -1 )
            $('#vue').change().parent().show();
          else
            $('#vue, #largeur, #format').parent().hide();
          // Mise à jour de l'aperçu
          majapercu();
        });
        // Actions sur #texte
        $('#texte').on("change keypress", function (e) {
          if ( e.which == 0 )
            return;
          if ( e.which == 13 )
            $('#fenetre a.icon-ok').click();
          // Si vide : modification automatique
          if ( this.value.length == 0 )  {
            this.setAttribute('data-auto',1);
            $('#doc').change();
          }
          else  {
            this.setAttribute('data-auto',0);
            majapercu();
          }
        });
        // Actions sur #vue
        $('#vue').on("change", function () {
          if ( $('#vue').is(':checked') ) {
            if ( $('#doc option:selected').text().slice(-4,-1) == 'pdf' ) {
              $('#largeur, #format').parent().show();
              $('#texte').parent().hide();
            }
            else if ( 'jpgpegpng'.indexOf($('#doc option:selected').text().slice(-4,-1)) > -1 ) {
              $('#largeur').parent().show();
              $('#format, #texte').parent().hide();
            }
          }
          else {
            $('#texte').parent().show();;
            $('#largeur, #format').parent().hide();
          }
          majapercu();
        });
        // Actions sur #format
        $('#format').on("change keyup", function (e) {
          if ( e.which == 13 )
            $('#fenetre a.icon-ok').click();
          majapercu();
        });
        // Actions sur #largeur
        $('#largeur').on("keydown", function (e) {
          if ( e.which == 38 )
            ++this.value;
          else if ( e.which == 40 )
            --this.value;
        }).on("change keyup", function (e) {
          if ( e.which == 0 )
            return;
          if ( e.which == 13 )
            $('#fenetre a.icon-ok').click();
          if ( this.value != this.getAttribute('data-valeur') ) {
            this.setAttribute('data-valeur', this.value);
            majapercu();
          }
        }).attr('data-valeur',100);
        // Actions sur #rep
        $('#rep').on('change',function () {
          $('#doc').html(data.docs[this.value]).change();
        });
        // Actions sur #mat
        $('#mat').on('change',function () {
          $('#rep').html(data.reps[this.value]).change();
        }).focus().change();
        // Insertion
        $('#fenetre a.icon-ok').on('click', function () {
          if ( $('#doc').val() )  {
            if ( $('#vue').is(':checked') && ( 'pdfjpgpegpng'.indexOf($('#doc option:selected').text().slice(-4,-1)) > -1 ) )
              insert(el,$('#apercu').html(),'','');
            else
              insert(el,'<a href="download?id='+$('#doc').val()+'">','</a>',$('#texte').val());
            $('#fenetre,#fenetre_fond').remove();
          }
        });
        // Sélection a priori d'une matière (programme de colle, cahier de texte,
        // pages associées à une matière)
        $('#mat option').each( function() {
          if ( $('body').attr('data-matiere') == this.value )
            $('#mat').val(this.value).change();
        });
      }
      
      // Si besoin de connection (mais ne pas lancer la fonction afficher_login)
      if ( data['etat'] == 'login_' ) {
        $('#fenetre > div:first').html('<p>Vous n\'êtes actuellement plus connecté. Vous devez vous identifier à nouveau pour récupérer la liste des répertoires et documents disponibles.</p>\
        <form>\
        <p class="ligne"><label for="login">Identifiant&nbsp;: </label><input type="text" name="login" id="login"></p>\
        <p class="ligne"><label for="motdepasse">Mot de passe&nbsp;: </label><input type="password" name="motdepasse" id="motdepasse"></p>\
        </form>'); 
        $('#fenetre input').on('keypress',function (e) {
          if (e.which == 13)
            $('#fenetre a.icon-ok').click();
        });
        $('#fenetre a.icon-ok').on('click', function () {
          $.ajax({url: 'ajax.php',
                  method: "post",
                  data: { login: $('#login').val(), motdepasse: $('#motdepasse').val(), recupdoc: '' },
                  dataType: 'json'})
            .success( function(data) {
              // Si erreur d'identification, on reste bloqué là
              if ( data['etat'] == 'login_nok' )
                $('#fenetre > div:first > p:first').html(data['message']).addClass('warning');
              // Sinon et si l'identification est ok, on y va
              else if ( data['etat'] == 'ok_' )
                affichedocs(data);
          });
        });
      }
      // Si retour de récupération ok
      else if ( data['etat'] == 'ok_' )
        affichedocs(data);
  });
}

function insertion_lien2(el) {
  popup('<a class="icon-ok" title="Valider"></a><h3>Insertion d\'un lien</h3>\
  <p class="ligne"><label for="texte">Texte visible&nbsp;: </label><input type="text" id="texte" value="'+marqueselection(el)+'" size="80"></p>\
  <p class="ligne"><label for="url">Adresse&nbsp;: </label><input type="text" id="url" value="http://" size="80"></p>\
  <hr><h3>Aperçu</h3><div id="apercu" style="text-align:center;"></div>',true);
  // Modification automatique de l'aperçu
  $('#fenetre input').on("click keyup", function () {
    $('#apercu').html( ( $('#texte').val().length ) ? '<a onclick="return false;" href="'+$('#url').val()+'">'+$('#texte').val()+'</a>' : '[Écrivez un texte visible]');
  }).on("keypress",function(e) {
    if ( e.which == 13 )
      $('#fenetre a.icon-ok').click();
  }).first().keyup().focus();
  // Insertion
  $('#fenetre a.icon-ok').on("click",function () {
    insert(el,'<a href="'+$('#url').val()+'">','</a>',$('#texte').val());
    $('#fenetre,#fenetre_fond').remove();
  });
  // Suppression des attributs liés à la sélection lors de la fermeture
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
}

function insertion_tex(el) {
  // Chargement de MathJax si non déjà chargé
  var chargement = ( typeof MathJax == 'undefined' ) ? '<script type="text/javascript" src="/MathJax/MathJax.js?config=TeX-AMS-MML_HTMLorMML"></script>\
<script type="text/x-mathjax-config">MathJax.Hub.Config({tex2jax:{inlineMath:[["$","$"],["\\\\(","\\\\)"]]}});</script>' : '';
  // Récupération de la sélection et du type de formule éventuelle
  var sel = marqueselection(el);
  var type = 't1';
  if ( sel.length )
    switch ( sel.substring(0,2) ) {
      case '\\[' :
      case '$$'  : type = 't2';
      case '\\(' : sel = sel.substring(2,sel.length-2); break;
      default    : sel = sel.trim('$');
    }
  // Affichage de la fenêtre d'édition/aperçu
  popup(chargement+'<a class="icon-montre" title="Mettre à jour l\'aperçu"></a><a class="icon-ok" title="Valider"></a><h3>Insertion de formules LaTeX</h3>\
  <p>Vous pouvez ci-dessous entrer et modifier une formule LaTeX. L\'aperçu présent en bas sera mis à jour uniquement lorsque vous cliquez sur l\'icône <span class="icon-montre"></span>.</p>\
  <p class="ligne"><label for="t1">La formule est en ligne (pas de retour)</label><input type="radio" name="type" id="t1" value="1"></p>\
  <p class="ligne"><label for="t2">La formule est hors ligne (formule centrée)</label><input type="radio" name="type" id="t2" value="2"></p>\
  <textarea id="formule" rows="3">'+sel+'</textarea>\
  <hr><h3>Aperçu</h3><div id="apercu" style="text-align:center;">[Demandez l\'aperçu en cliquant sur l\'icône <span class="icon-montre"></span>]</div>',true);
  $('#'+type).prop("checked",true);
  $('#formule').focus();
  // Mise à jour de l'aperçu
  $('#fenetre a.icon-montre').on("click", function () {
    if ( $('#formule').val().length ) {
      $('#apercu').html( ( $('#t1').is(":checked") ) ? '$'+$('#formule').val()+'$' : '\\['+$('#formule').val()+'\\]').css('text-align','left');
      MathJax.Hub.Queue(["Typeset",MathJax.Hub,"apercu"]);
    }
    else
      $('#apercu').html('[Écrivez une formule]').css('text-align','center');
  });
  // Insertion
  $('#fenetre a.icon-ok').on("click",function () {
    if ( $('#t1').is(":checked") )
      insert(el,'$','$',$('#formule').val());
    else
      insert(el,'\\[','\\]',$('#formule').val());
    $('#fenetre,#fenetre_fond').remove();
  });
  // Suppression des attributs liés à la sélection lors de la fermeture
  $('#fenetre a.icon-ferme,#fenetre_fond').on("click",function () {
    marqueselection(el,true);
  });
}

// Fonctions lancées par les boutons à insertion directe
function insertion_par1(el)    { insert(el,'<p>','</p>'); }
function insertion_par2(el)    { insert(el,'<div class=\'note\'>','</div>'); }
function insertion_par3(el)    { insert(el,'<div class=\'annonce\'>','</div>'); }
function insertion_retour(el)  { insert(el,'<br>',''); }
function insertion_gras(el)    { insert(el,'<strong>','</strong>'); }
function insertion_italique(el) { insert(el,'<em>','</em>'); }
function insertion_souligne(el) { insert(el,'<u>','</u>'); }
function insertion_exp(el)     { insert(el,'<sup>','</sup>'); }
function insertion_ind(el)     { insert(el,'<sub>','</sub>'); }

// Aide
function aidetexte() {
  popup('<h3>Aide et explications</h3>\
  <p>Il y a deux modes d\'éditions possibles pour éditer un texte&nbsp;: le mode «&nbsp;balises visibles&nbsp;» et le mode «&nbsp;balises invisibles&nbsp;». Il est possible de passer de l\'un à l\'autre&nbsp;:</p>\
  <ul>\
    <li><span class="icon-source"></span> permet de passer en mode «&nbsp;balises visibles&nbsp;» (par défaut), où le texte à taper est le code HTML de l\'article. Ce mode est plus précis. Les boutons aux dessus aident à utiliser les bonnes balises.</li>\
    <li><span class="icon-nosource"></span> permet de passer en mode «&nbsp;balises invisibles&nbsp;», où le texte est tel qu\'il sera affiché sur la partie publique, et modifiable. Ce mode est moins précis, mais permet le copié-collé depuis une page web ou un document Word/LibreOffice.\
  </ul>\
  <p>Une fonction de nettoyage du code HTML, permettant d\'assurer une homogénéité et une qualité d\'affichage optimales, est lancée à chaque commutation entre les deux modes, à chaque clic sur un des boutons disponibles, à chaque copie/coupe de texte et à chaque passage à la ligne.</p>\
  <p>En HTML, toutes les mises en formes sont réalisées par un encadrement de texte entre deux balises&nbsp;: &lt;h3&gt; et &lt;/h3&gt; pour un gros titre, &lt;p&gt; et &lt;/p&gt; pour un paragraphe. Le retour à la ligne simple, qui ne doit exister que très rarement, est une balise simple &lt;br&gt;. Mais les boutons disponibles sont là pour vous permettre de réaliser le formattage que vous souhaitez&nbsp;:</p>\
  <ul>\
    <li><span class="icon-titres"></span>&nbsp;: différentes tailles de titres (fenêtre supplémentaire pour choisir)</li>\
    <li><span class="icon-par1"></span>&nbsp;: paragraphe classique, qui doit obligatoirement encadrer au minimum chaque ligne de texte. Apparaît automatiquement au passage à la ligne si on l\'oublie.</li>\
    <li><span class="icon-par2"></span>&nbsp;: paragraphe important, écrit en rouge</li>\
    <li><span class="icon-par3"></span>&nbsp;: paragraphe très important, écrit en rouge et encadré</li>\
    <li><span class="icon-retour"></span>&nbsp;: retour à la ligne. Identique à un appui sur Entrée, et souvent inutile.</li>\
    <li><span class="icon-gras"></span>&nbsp;: mise en gras du texte entre les balises</li>\
    <li><span class="icon-italique"></span>&nbsp;: mise en italique du texte entre les balises</li>\
    <li><span class="icon-souligne"></span>&nbsp;: soulignement du texte entre les balises</li>\
    <li><span class="icon-omega"></span>&nbsp;: lettres grecques (fenêtre supplémentaire pour choisir)</li>\
    <li><span class="icon-sigma"></span>&nbsp;: symboles mathématiques (fenêtre supplémentaire pour choisir)</li>\
    <li><span class="icon-exp"></span>&nbsp;: mise en exposant du texte entre les balises</li>\
    <li><span class="icon-ind"></span>&nbsp;: mise en indice du texte entre les balises</li>\
    <li><span class="icon-ol"></span>&nbsp;: liste numérotée. Une fenêtre supplémentaire permet de choisir le type (1,A,a,I,i) et la première valeur. Les différentes lignes de la liste sont constituées par les balises &lt;li&gt; et &lt;/li&gt;</li>\
    <li><span class="icon-ul"></span>&nbsp;: liste à puces. Les différentes lignes de la liste sont constituées par les balises &lt;li&gt; et &lt;/li&gt;</li>\
    <li><span class="icon-lien1"></span>&nbsp;: lien d\'un document disponible ici (fenêtre supplémentaire pour choisir)</li>\
    <li><span class="icon-lien2"></span>&nbsp;: lien vers un autre site web (fenêtre supplémentaire pour entre l\'adresse)</li>\
    <li><span class="icon-tex"></span>&nbsp;: insertion de code LaTeX (fenêtre supplémentaire pour le taper)</li>\
  </ul>\
  <p class="tex2jax_ignore">Il est possible d\'insérer du code en LaTeX, sur une ligne séparée (balises \\[...\\] ou balises $$...$$) ou au sein d\'une phrase (balises $...$ ou balises \\(...\\)). Il faut ensuite taper du code en LaTeX à l\'intérieur. La prévisualisation est réalisée en direct.</p>',false);
} 

///////////////////////////////
// Modification des articles //
///////////////////////////////

// Échange vertical de deux éléments
// el1 doit être le plus haut des deux
function echange(el1,el2) {
  if ( el1.length && el2.length ) {
    $('article').css('position','relative');
    el1.css('opacity',0.3); el2.css('opacity',0.3);
    el2.animate({top: el1.position().top-el2.position().top},1000);
    el1.animate({top: (el2.outerHeight(true)+el2.outerHeight())/2},1000,function() {
      el1.css('opacity',1); el2.css('opacity',1);
      el1.insertAfter(el2);
      el1.css({'position': 'static', 'top': 0});
      el2.css({'position': 'static', 'top': 0});
    });
  }
}

// Disparition de la partie publique
function cache(el) {
  var prop = el.parent().attr('data-id').split('|');
  $.ajax({url: 'ajax.php',
          method: "post",
          data: { cache:1, table:prop[0], id:prop[1] },
          dataType: 'json',
          el: el,
          fonction: function(el) {
            el.parent().addClass('cache');
            el.removeClass('icon-cache').addClass('icon-montre').off("click").on("click", function() {
              montre($(this));
            }).attr("title","Montrer à nouveau");
          }
  });
}

// Apparition sur la partie publique
function montre(el) {
  var prop = el.parent().attr('data-id').split('|');
  $.ajax({url: 'ajax.php',
          method: "post",
          data: { montre:1, table:prop[0], id:prop[1] },
          dataType: 'json',
          el: el,
          fonction: function(el) {
            el.parent().removeClass('cache');
            el.removeClass('icon-montre').addClass('icon-cache').off("click").on("click", function() {
              cache($(this));
            }).attr("title","Cacher à nouveau");
          }
  });
}

// Montée
function monte(el) {
  var parent = el.parent();
  var prop = parent.attr('data-id').split('|');
  $.ajax({url: 'ajax.php',
          method: "post",
          data: { monte:1, table:prop[0], id:prop[1] },
          dataType: 'json',
          el: parent,
          fonction: function(el) {
            if ( !(el.prev().prev().is('article')) ) {
              el.children('.icon-monte').hide(1000);
              el.prev().children('.icon-monte').show(1000);
            }
            if ( !(el.next().is('article')) ) {
              el.children('.icon-descend').show(1000);
              el.prev().children('.icon-descend').hide(1000);
            }
            echange(el.prev(),el);
          }
  });
}

// Descente
function descend(el) {
  var parent = el.parent();
  var prop = parent.attr('data-id').split('|');
  $.ajax({url: 'ajax.php',
          method: "post",
          data: { descend:1, table:prop[0], id:prop[1] },
          dataType: 'json',
          el: parent,
          fonction: function(el) {
            if ( !(el.prev().is('article')) ) {
              el.children('.icon-monte').show(1000);
              el.next().children('.icon-monte').hide(1000);
            }
            if ( !(el.next().next().is('article')) ) {
              el.children('.icon-descend').hide(1000);
              el.next().children('.icon-descend').show(1000);
            }
            echange(el,el.next());
          }
  });
}

// Suppression
function supprime(el) {
  var parent = el.parent();
  popup('<h3>Confirmer la demande de suppression</h3><p class="suppression"><button class="icon-ok" title="Confirmer la suppression"></button>&nbsp;&nbsp;&nbsp;<button class="icon-annule" title="Sortir sans supprimer"></button></p>',true);
  $('#fenetre .icon-ok').on("click",function () {
    $('#fenetre,#fenetre_fond').remove();
    var prop = parent.attr('data-id').split('|');
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { supprime:1, table:prop[0], id:prop[1] },
            dataType: 'json',
            el: parent,
            fonction: function(el) {
              if ( prop[0] == 'colles' ) {
                el.removeClass('cache');
                el.children('.icon-cache,.icon-montre,.icon-supprime').remove();
                el.children('.icon-ajoute').show();
                el.children('div.editable').each( function() {
                  this.setAttribute('class',this.getAttribute('class').replace('editable','editable-remplace'));
                }).html('<p>Le programme de colles de cette semaine n\'est pas encore défini.</p>');
              }
              else if ( prop[0] == 'utilisateurs' )
                location.reload(true);
              else
                el.remove();
            }
    });
  });
  $('#fenetre .icon-annule').on("click",function () {
    $('#fenetre,#fenetre_fond').remove();
  });
}

// Formulaire généré lors des clics sur les boutons icon-ajoute et icon-prefs
function formulaire(el) {
  var action = el.getAttribute('data-id');
  // Si pas d'action : validation de compte utilisateur
  if ( !action ) {
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { table:'utilisateurs', id:$(el).parent().attr('data-id').split('|')[1] },
            dataType: 'json',
            el: el,
            fonction: function(el) {
              location.reload(true);
            }
    });
    return true;
  }
  // Si l'attribut data-remplace est fourni, il s'agit de l'identifiant de
  // l'article qui sera modifié après validation 
  if ( el.hasAttribute('data-remplace') ) {
    // Modification de l'article et insertion du formulaire
    var article = $('article#'+el.getAttribute('data-remplace'));
    article.children('.icon-ajoute').after('<a class="icon-annule" title="Annuler"></a><a class="icon-ok" title="Valider"></a>');
    article.children('[class*="-remplace"],.icon-ajoute').hide();
    var form = $('<form></form>').appendTo(article).html($('#form-'+action).html());
    // Modification de l'identifiant à envoyer
    form.children('[name="id"]').val(el.getAttribute('data-remplace'));
  }
  // Cas sans remplacement : insertion en haut de page
  else {
    // Suppression d'un éventuel contenu épinglé existant
    $('#epingle').remove();
    // Création du nouveau formulaire
    var article = $('<article id="epingle"><a class="icon-ferme" title="Fermer"></a>\
  <a class="icon-aide" title="Aide pour ce formulaire"></a>\
  <a class="icon-ok" title="Valider"></a></article>').insertBefore('article:first');
    var form = $('<form></form>').appendTo(article).html($('#form-'+action).html());
  }
  // Facilités
  form.find('.edithtml').textareahtml();
  form.find('[data-placeholder]').placeholder();
  if ( form.children('[name="table"]').val() == 'cdt' )
    form.boutonscdt();
  else if ( form.children('[name="table"]').val() == 'cdt-seances' )
    form.affichechamps();
  else if ( $('table#notes').length )
    notes(el);
  // Édition des élèves d'un nouveau groupe
  form.find('.usergrp > .icon-edite').on("click", function() {
    utilisateursgroupe(this);
  });
  // Actions des boutons fermeture, annulation, aide, envoi
  article.children('.icon-ferme').on("click",function() {
    $('#epingle').remove();
  });
  article.children('.icon-annule').on("click",function() {
    article.children('form,.icon-annule,.icon-ok').remove();
    article.children('[class*="-remplace"],.icon-ajoute').show();
  });
  article.children('a.icon-aide').on("click", function() {
    popup($('#aide-'+action).html(),false);
  });
  article.children('a.icon-ok').on("click", function() {
    // Nettoyage et synchronisation si besoin
    form.children('.edithtml').each( function() {
      this.value = nettoie( ( $(this).is(':visible') ) ? this.value : $(this).next().html() );
    });
    // Nettoyage des notes à ne pas mettre
    if ( form.children('[name="table"]').val() == 'notes' )
      $('#epingle select:not(:visible)').val('x');
    // Envoi
    $.ajax({url: 'ajax.php',
            method: "post",
            data: form.serialize(),
            dataType: 'json',
            el: article,
            fonction: function(el) {
              // Sans remplacement : on recharge la page
              if ( el.attr('id') == 'epingle' ) {
                // Cas du réglage des demandes de création de compte
                // (utilisateurs.php) : ne rien faire
                if ( !$('#epingle #autoriser').length )
                  location.reload(true);
              }
              // Avec remplacement : on modifie l'article pour permettre à nouveau l'édition
              else {
                el.children('.icon-supprime').show();
                if ( el.find('[name="cache"]').is(":checked") ) {
                  el.children('.icon-montre').show();
                  el.addClass('cache');
                }
                else
                  el.children('.icon-cache').show();
                el.children('[class*="-remplace"]').show().each( function() {
                  this.setAttribute('class',this.getAttribute('class').replace('-remplace',''));
                  var id = this.getAttribute('data-id');
                  $(this).html(article.find('[name="'+id.slice(id.indexOf('|')+1,id.lastIndexOf('|'))+'"]').val());
                });
                el.children('.editable').editinplace();
                el.children('.icon-annule,.icon-ok,form').remove();
              }
            }
    });
  });
  // Envoi par appui sur Entrée
  form.find('input,select').on('keypress',function (e) {
    if ( e.which == 13 ) {
      e.preventDefault();
      article.children('a.icon-ok').click();
    }
  });
}

// Envoi automatique : pour les formulaire déjà présents
function valide(el) {
  var data = '';
  // Envoi de courriel
  if ( $('#mail').length ) {
    // Pas d'envoi si pas de destinataire ou si sujet vide
    if ( $('.editabledest').children('span').text() == '[Personne]' )
      affiche('Il faut au moins un destinataire pour envoyer le courriel.','nok');
    else if ( !$('[name="sujet"]').val().length )
      affiche('Il faut un sujet non vide pour envoyer le courriel.','nok');
    else
      data = $('#mail').serialize();
  }
  // Modification de planning
  else if ( $('#planning').length ) {
    data = $('form').serialize();
  }
  else {
    var form = $(el).parent();
    var id = form.parent().attr('data-id').split('|');
    data = 'table='+id[0]+'&id='+id[1]+'&'+form.serialize();
  }
  if ( data.length )
    $.ajax({url: 'ajax.php',
            method: "post",
            data: data,
            dataType: 'json',
            el: el,
            fonction: function(el) {
              if ( !$(el).is('[data-noreload]') )
                location.reload(true);
            }
    });
}

////////////////////
// Autres actions //
////////////////////

// Suppression massive des éléments d'une matière
function suppressionmultiple(el) {
  popup('<h3>Confirmer la demande de suppression</h3><p class="suppression"><button class="icon-ok" title="Confirmer la suppression"></button>&nbsp;&nbsp;&nbsp;<button class="icon-annule" title="Sortir sans supprimer"></button></p>',true);
  $('#fenetre .icon-ok').on("click",function () {
    var prop = el.getAttribute('data-id').split('|');
    $('#fenetre,#fenetre_fond').remove();
    $.ajax({url: 'ajax.php',
            method: "post",
            data: 'table='+prop[0]+'&id='+prop[1]+'&supprime_'+prop[2]+'=1',
            dataType: 'json',
            el: $(el),
            fonction: function(el) {
              el.remove();
            }
    });
  });
  $('#fenetre .icon-annule').on("click",function () {
    $('#fenetre,#fenetre_fond').remove();
  });
}

// Édition des utilisateurs associés à une matière
function utilisateursmatiere(el) {
  var bouton = el;
  var matiere = bouton.getAttribute('data-matiere').split('|'); // 0: nom, 1: id
  popup($('#form-utilisateurs').html(),true);
  // Modification de la matière indiquée
  $('#fenetre').addClass('usermat').children('h3').append(matiere[0]);
  // Ajout des bons id pour les checkboxes et envoi des données
  $('#fenetre :checkbox').attr('id',function() { return this.name; }).on('change',function() {
    var box = this;
    var id = box.id.substr(1);
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { table:'utilisateurs', id:id, matiere:matiere[1], ok:(box.checked?1:0) },
            dataType: 'json',
            el: box,
            fonction: function(el) {
              // Mise à jour de la liste des utilisateurs
              if ( el.checked ) {
                utilisateurs = utilisateurs+','+id;
                $(box).prev().addClass('labelchecked');
              }
              else {
                utilisateurs = (','+utilisateurs+',').replace(','+id+',',',').slice(1,-1);
                $(box).prev().removeClass('labelchecked');
              }
              bouton.setAttribute('data-uid',utilisateurs);
              // Mise à jour du descriptif de la matière
              var np = $('#fenetre .a4:checked').length;
              var nc = $('#fenetre .a3:checked').length;
              var ne = $('#fenetre .a2:checked').length;
              var ni = $('#fenetre .a1:checked').length;
              var n = np+nc+ne+ni;
              if ( n )  {
                var u = '';
                if ( np )  u += ', '+np+' professeur'+(np>1?'s':'');
                if ( nc )  u += ', '+nc+' colleur'+(nc>1?'s':'');
                if ( ne )  u += ', '+ne+' élève'+(ne>1?'s':'');
                if ( ni )  u += ', '+ni+' invité'+(ni>1?'s':'');
                $(bouton).parent().children('span').text('Cette matière concerne '+n+' utilisateur'+(n>1?'s':'')+' dont '+u.substr(1)+'.');
              }
              else
                $(bouton).parent().children('span').text('Cette matière ne concerne aucun utilisateur.');
            }
    }).success( function(data) {
      // Si erreur, on décoche
      if ( data['etat'] != 'ok' )
        box.checked = !box.checked;
    });
  });
  // Cochage des utilisateurs déjà associés
  var utilisateurs = bouton.getAttribute('data-uid');
  $('#u'+utilisateurs.replace(/,/g,',#u')).prop("checked",true);
  $('#fenetre :checked').prev().addClass('labelchecked');
}

// Édition des utilisateurs (élèves) associés à un groupe
function utilisateursgroupe(el) {
  var bouton = el;
  var article = $(el).parent().parent();
  popup($('#form-utilisateurs').html(),true);
  // Modification de la matière indiquée
  $('#fenetre').addClass('usergrp').children('h3').append(article.find('span.editable').text() || article.find('input').val());
  // Ajout des bons id pour les checkboxes
  $('#fenetre :checkbox').attr('id',function() { return this.name; });
  // Si article est en fait une div, on est dans le formulaire de nouveau groupe
  // donc mise à jour à la fin
  if ( article.is('div') ) {
    $('#fenetre :checkbox').on('change',function() {
      $(this).prev().toggleClass('labelchecked',el.checked);
    });
    $('<a class="icon-ok" title="Valider"></a>').insertAfter('#fenetre .icon-ferme').on("click",function() {
      var utilisateurs = $('#fenetre input:checked').map(function() { return this.id.replace('u',''); }).get().join();
      bouton.setAttribute('data-uid',utilisateurs);
      $(bouton).parent().next().val(utilisateurs);
      $(bouton).parent().children('span').text($('#fenetre input:checked').parent().map(function() {
        return $(this).text().replace(' (identifiant)','').trim();
      }).get().join(', '));
      $('#fenetre, #fenetre_fond').remove();
    });
  }
  // Sinon envoi des données à chaque fois
  else {
    $('#fenetre :checkbox').on('change',function() {
      var box = this;
      var eleve = box.id.substr(1);
      $.ajax({url: 'ajax.php',
              method: "post",
              data: { table:'groupes', id:article.attr('data-id').split('|')[1], eleve:eleve, ok:(box.checked?1:0) },
              dataType: 'json',
              el: box,
              fonction: function(el) {
                // Mise à jour de la liste des utilisateurs
                if ( el.checked ) {
                  utilisateurs = utilisateurs+','+eleve;
                  $(box).prev().addClass('labelchecked');
                }
                else {
                  utilisateurs = (','+utilisateurs+',').replace(','+eleve+',',',').slice(1,-1);
                  $(box).prev().removeClass('labelchecked');
                }
                bouton.setAttribute('data-uid',utilisateurs);
                // Mise à jour du descriptif du groupe
                $(bouton).parent().children('span').text($('#fenetre input:checked').parent().map(function() {
                  return $(this).text().replace(' (identifiant)','').trim();
                }).get().join(', '));
              }
      }).success( function(data) {
        // Si erreur, on décoche
        if ( data['etat'] != 'ok' )
          box.checked = !box.checked;
      });
    });
  }
  // Cochage des utilisateurs déjà associés
  var utilisateurs = bouton.getAttribute('data-uid');
  $('#u'+utilisateurs.replace(/,/g,',#u')).prop("checked",true);
  $('#fenetre :checked').prev().addClass('labelchecked');
}

// Édition des utilisateurs destinataires d'un courriel
function utilisateursmail(el) {
  popup($('#form-utilisateurs').html(),true);
  // Ajout des identifiants pour utilisation des labels
  $('#fenetre [name="dest[]"]').each( function() {
    $(this).attr('id','u'+this.value);
  });
  // Sélection automatique des utilisateurs déjà choisis
  var ids = $('[name="id-copie"]').val().split(',');
  for(var i=0; i<ids.length; i++)
    $('#fenetre [name="dest[]"][value="'+ids[i]+'"]').prop("checked",true);
  ids = $('[name="id-bcc"]').val().split(',');
  for(var i=0; i<ids.length; i++)
    $('#fenetre [name="dest_bcc[]"][value="'+ids[i]+'"]').prop("checked",true);
  // Bouton de sélection multiple
  $('#fenetre .icon-cocher').on("click keyup", function () {
    var classe = this.getAttribute('data-classe');
    $('#fenetre .'+classe+':not(:disabled)').prop("checked",true);
    $(this).hide();
    $(this).next().show();
    classe = ( classe.indexOf('bcc') > 0 ) ? classe.replace('bcc','c') : classe.replace('c','bcc');
    $('#fenetre .'+classe+':not(:disabled)').prop("checked",false);
    $('#fenetre .icon-cocher[data-classe="'+classe+'"]').show();
    $('#fenetre .icon-decocher[data-classe="'+classe+'"]').hide();
  });
  $('#fenetre .icon-decocher').on("click keyup", function () {
    $('#fenetre .'+this.getAttribute('data-classe')+':not(:disabled)').prop("checked",false);
    $(this).hide();
    $(this).prev().show();
  }).hide();
  // Décochage automatique de l'autre case du même utilisateur
  $('#fenetre :checkbox[name]').on("change", function () {
    if( $(this).is(':checked') )
      $('#fenetre :checkbox[name][value="'+this.value+'"][name!="'+this.name+'"]').prop("checked",false);
  });
  // Groupes
  $('#fenetre :checkbox:not([name])').on("click", function () {
    var ids = this.value.split(',');
    for (var i=0; i<ids.length; i++)
      $('#fenetre :checkbox[name="'+this.className+'[]"][value="'+ids[i]+'"]').prop("checked",$(this).prop('checked')).change();
    if ( $(this).is(':checked') )
      $('#fenetre :checkbox[class="'+((this.className=='dest')?'dest_bcc':'dest')+'"][value="'+this.value+'"]').prop("checked",false);
  });
  // Récupération des valeurs
  $('#fenetre .icon-ok').on("click", function() {
    $('[name="id-copie"]').val(
      $('#fenetre [name="dest[]"]:checked').map(function() { return this.value; }).get().join(',')
    );
    $('[name="id-bcc"]').val(
      $('#fenetre [name="dest_bcc[]"]:checked').map(function() { return this.value; }).get().join(',')
    );
    $(el).prev().text(
      $('#fenetre [name="dest[]"]:checked').parent().prev().map(function() { return $(this).text().replace(' (identifiant)',''); }).get()
      .concat(
        $('#fenetre [name="dest_bcc[]"]:checked').parent().prev().prev().map(function() { return $(this).text().replace(' (identifiant)','')+' (CC)'; }).get()
      ).join(', ')
    )
    if ( !$(el).prev().text().length )
      $(el).prev().text('[Personne]');
    $('#fenetre, #fenetre_fond').remove();
  });
}

// Édition des notes (on est déjà passé par la fonction formulaire)
function notes(el)  {
  // Clic sur les groupes
  $('#epingle :checkbox').on("click", function () {
    // Récupération des élèves du groupe
    var ids = $('#epingle .grpnote:checked').map(function() { return this.value.split(','); }).get().concat();
    // On cache tous les élèves sauf ceux ayant déjà une note pour l'édition
    // de notes déjà saisies (lignes repérées par data-orig=true)
    $('#epingle tr[data-id]:not([data-orig])').hide();
    for (var i=0; i<ids.length; i++)
      $('#epingle tr[data-id="'+ids[i]+'"]').show();
  });
  // Génération des select de notes
  $('#epingle tr[data-id]').each( function() {
    $(this).children('td:last').html($('#epingle div').html());
    $(this).find('select').attr('name','e'+$(this).attr('data-id'));
  });
  $('#epingle div').remove();
  // Modification initiale
  // On cache tous les élèves a priori s'il y a des groupes
  if ( $('#epingle :checkbox').length )
    $('#epingle tr[data-id]').hide();
  // Si édition de notes, on affiche les notes déjà mises
  if ( el.getAttribute('data-eleves') ) {
    // Titre et identifiant de la semaine 
    $('#epingle h4').text($(el).parent().children('h3').text());
    $('#epingle [name="id"]').val($(el).parent().attr('data-id').split('|')[1]);
    // Récupération des élèves et des notes
    var eleves = el.getAttribute('data-eleves').split('|');
    var notes = el.getAttribute('data-notes').split('|');
    // Modification et affichage. Les lignes sont marquées data-orig=true pour
    // rester affichées lors du clic sur une checkbox de groupe, mais ce
    // marquage est supprimé à la première modification de la note
    for (var i=0; i<eleves.length; i++)
      $('#epingle tr[data-id="'+eleves[i]+'"]').attr('data-orig',true).show().find('select').val(notes[i]).on('change',function() {
        $(this).parent().parent().removeAttr('data-orig');
      });
    // Impossible de modifier les élèves ayant déjà une note
    var dn = dejanotes[$('#epingle [name="id"]').val().split('-')[0]].split(',');
    for (i=0; i<dn.length; i++)
      $('#epingle tr[data-id="'+dn[i]+'"]:not(:visible)').addClass('dejanote').find('select').prop('disabled',true);
    $('.dejanote td:first-child').each( function() {
      $(this).text($(this).text()+' (noté par un autre colleur)');
    });
  }
  else {
    $('#epingle #semaine').on("change keyup", function() {
      var semaine = $(this).val().split('-')[0];
      // Récupération des "déjànotés" précédents
      $('.dejanote td:first-child').each( function() {
        $(this).text($(this).text().replace(' (noté par un autre colleur)',''));
      });
      $('.dejanote').removeClass('dejanote').find('select').prop('disabled',false);
      // Impossible de modifier les élèves ayant déjà une note
      if ( semaine > 0 ) {
        var dn = dejanotes[semaine].split(',');
        for (var i=0; i<dn.length; i++)
          $('#epingle tr[data-id="'+dn[i]+'"]').addClass('dejanote').find('select').prop('disabled',true);
        $('.dejanote td:first-child').each( function() {
          $(this).text($(this).text()+' (noté par un autre colleur)');
        });
      }
    });
  }
}

// Envoi des courriels
function envoimail(el) {
  // Pas d'envoi si pas de destinataire
  if ( $('.editabledest').children('span').text() == '[Personne]' )
    affiche('Il faut au moins un destinataire pour envoyer le courriel.','nok');
  else
  $.ajax({url: 'ajax.php',
          method: "post",
          data: $('#mail').serialize(),
          dataType: 'json',
          el: '',
          fonction: function(el) {
            location.reload(true);
          }
  });
}

// Édition des répertoires
function modifierepertoire(el) {
  var action = ( el.className == 'icon-edite' ) ? 'repertoire' : 'ajouterep';
  // Suppression d'un éventuel contenu épinglé existant
  $('#epingle').remove();
  // Création du nouveau formulaire
  var article = $('<article id="epingle"><a class="icon-ferme" title="Fermer"></a>\
  <a class="icon-aide" title="Aide pour ce formulaire"></a>\
  <a class="icon-ok" title="Valider"></a></article>').insertAfter('#parentsdoc');
  var form = $('<form onsubmit="return false;"></form>').appendTo(article).html($('#form-'+action).html());
  if ( action == 'repertoire' ) {
    // Identifiant
    var id = $(el).parent().attr('data-id').split('|')[1];
    // Donnees : parent,menu,protection
    var donnees = $(el).parent().attr('data-donnees').split('|');
    // Remplissage du formulaire par les données du répertoire
    // split(/\/\s/) : regexp = "/"+"espace" ("&nbsp;" passé dans text())
    var nom = $(el).parent().children('.nom').text().split(/\/\s/).pop() || $(el).parent().find('input').val();
    form.find('em').text(nom);
    if ( donnees[0] == 0 ) {
      form.find('#nom,#parent,#menu').parent().remove();
    }
    else {
      form.find('#nom').val(nom);
      form.find('[data-parents*=",'+id+',"]').prop('disabled', true);
      if ( donnees[1] == '1' )
        form.find('#menu').prop('checked',true);
    }
    $('#protection option[value="'+donnees[2]+'"]').prop('selected', true);
    form.find('[name="id"]').val(id);
  }
  // Actions des boutons fermeture, annulation, aide, envoi
  article.children('.icon-ferme').on("click",function() {
    $('#epingle').remove();
  });
  article.children('a.icon-aide').on("click", function() {
    popup($('#aide-'+action).html(),false);
  });
  article.children('a.icon-ok').on("click", function() {
    // Envoi
    $.ajax({url: 'ajax.php',
            method: "post",
            data: form.serialize(),
            dataType: 'json',
            el: article,
            fonction: function(el) {
              location.reload(true);
            }
    });
  });
  // Envoi par appui sur Entrée
  form.find('input,select').on('keypress',function (e) {
    if ( e.which == 13 )
      article.children('a.icon-ok').click();
  });
}

// Édition des documents
function modifiedocument(el) {
  var action = ( el.className == 'icon-edite' ) ? 'document' : 'ajoutedoc';
  // Suppression d'un éventuel contenu épinglé existant
  $('#epingle').remove();
  // Création du nouveau formulaire
  var article = $('<article id="epingle"><a class="icon-ferme" title="Fermer"></a>\
  <a class="icon-aide" title="Aide pour ce formulaire"></a>\
  <a class="icon-ok" title="Valider"></a></article>').insertAfter('#parentsdoc');
  form = $('<form onsubmit="return false;"></form>').appendTo(article).html($('#form-'+action).html());
  // Remplissage du formulaire par les données du document
  // Nom du fichier ou nom du répertoire parent pour l'ajout
  var nom = $(el).parent().children('.nom').text() || $(el).parent().find('input').val();
  form.find('em').text(nom);
  if ( action == 'document' ) {
    form.find('#nom').val(nom);
    $('#protection option[value="'+$(el).parent().attr('data-protection')+'"]').prop('selected', true);
    form.find('[name="id"]').val($(el).parent().attr('data-id').split('|')[1]);
  }
  else {
    $('#protection option[value="'+$(el).parent().attr('data-donnees').split('|')[2]+'"]').prop('selected', true);
    form.find('[name="parent"]').val($(el).parent().attr('data-id').split('|')[1]);
    // Modification automatique du champ nom si ajout d'un fichier
    form.find('#fichier').on('change',function () {
      if ( !form.find('#nom').val().length )
        var f = this.value;
        form.find('#nom').val(f.substring(f.lastIndexOf('\\')+1,f.lastIndexOf('.')) || f);
    });
  }
  // Actions des boutons fermeture, annulation, aide, envoi
  article.children('.icon-ferme').on("click",function() {
    $('#epingle').remove();
  });
  article.children('a.icon-aide').on("click", function() {
    popup($('#aide-'+action).html(),false);
  });
  article.children('a.icon-ok').on("click", function() {
    var data = new FormData(form[0]);
    // Envoi
    $.ajax({url: 'ajax.php',
            method: "post",
            data: data,
            dataType: 'json',
            contentType:false,
            processData:false,
            el: article,
            fonction: function(el) {
              location.reload(true);
            }
    });
  });
  // Envoi par appui sur Entrée
  form.find('input,select').on('keypress',function (e) {
    if ( e.which == 13 )
      article.children('a.icon-ok').click();
  });
}

///////////////////
// Requêtes AJAX //
///////////////////
$(document).ajaxSend( function(ev,xhr,settings) {
              $('body').css('cursor','wait');
              if ( settings.data.append ) // Pour les FormData
                settings.data.append('csrf-token',$('body').attr('data-csrf-token'));
              else
                settings.data = 'csrf-token='+$('body').attr('data-csrf-token')+'&'+settings.data;
            })
           .ajaxStop( function() {
              $('body').css('cursor','auto');
            })
           .ajaxSuccess( function(ev,xhr,settings) {
              var data = xhr.responseJSON;
              switch (data['etat']) {
                // Si ok, on l'affiche dans le log et on lance la "fonction" de mise à jour de l'affichage
                case 'ok':
                  affiche(data['message'],'ok');
                  settings.fonction(settings.el);
                  break;
                // Si non ok, on l'affiche dans le log et on ne fait rien de plus
                case 'nok':
                  affiche(data['message'],'nok');
                  break;
                // Si 'login' : il faut se reconnecter
                case 'login':
                  afficher_login(settings);
              }
            });

////////////////////////////////////////////////////////////////////////////
// Modification des éléments (nécessite le chargement complet de la page) //
////////////////////////////////////////////////////////////////////////////
$( function() {

  // Édition en place des éléments de classe "editable"
  $('.editable').editinplace();

  // Édition des propriétés des éléments des cahiers de texte
  $('.titrecdt').editinplacecdt();

  // Édition des propriétés des raccourcis des cahiers de texte
  $('.cdt-raccourcis').affichechamps();

  // Boutons de suppression massive des éléments de matière et des informations d'une page
  $('.supprmultiple').on("click", function() {
    suppressionmultiple(this);
  });

  // Édition des utilisateurs d'une matière
  $('.usermat .icon-edite').on("click", function() {
    utilisateursmatiere(this);
  });

  // Édition des utilisateurs destinataires d'un courriel
  $('.editabledest .icon-edite').on("click", function() {
    utilisateursmail(this);
  });
  
  // Édition de la préférence de copie de courriel
  $('[name="copie"]').on("change", function() {
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { table:'mailprefs', champ:'mailcopy', id:0, val:$(this).prop('checked')?1:0 },
            dataType: 'json',
            el: '',
            fonction: function(el) {
              return true;
            }
    });
  });

  // Édition des répertoires
  $('#parentsdoc .icon-edite, .rep > .icon-edite, .icon-ajouterep').on("click", function() {
    modifierepertoire(this);
  });

  // Édition des documents
  $('.doc > .icon-edite, .icon-ajoutedoc').on("click", function() {
    modifiedocument(this);
  });

  // Édition de l'utilisation des groupes d'élèves
  $('[name="mailnotes"]').on("change", function() {
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { table:'groupes', champ:'mailnotes', id:$(this).attr('id').substr(9), val:$(this).find('option:selected').val() },
            dataType: 'json',
            el: '',
            fonction: function(el) {
              return true;
            }
    });
  });

  // Édition des utilisateurs (élèves) d'un groupe
  $('.usergrp .icon-edite').on("click", function() {
    utilisateursgroupe(this);
  });

  // Div d'affichage des résultats des requêtes AJAX
  $('#log').hide().on("click", function() {
    $(this).hide();
  });

  // Boutons cache, montre, monte, descend, supprime
  $('a.icon-cache,a.icon-montre,a.icon-monte,a.icon-descend,a.icon-supprime').on("click", function() {
    window[this.className.substring(5)]($(this));
  });

  // Aide
  $('a.icon-aide').on("click", function() {
    popup($('#aide-'+this.getAttribute('data-id')).html(),false);
  });

  // Formulaires affichés par clic sur les icônes ajouts et préférences
  $('a.icon-prefs.general,a.icon-ajoute').on("click", function() {
    formulaire(this);
  });

  // Validation des formulaires déjà présents (pas pour les transformés)
  // Aussi utilisé pour l'envoi de courriel
  $('a.icon-ok').on("click", function() {
    valide(this);
  });

  // Déconnexion
  $('a.icon-deconnexion').on('click',function(e) {
    $.ajax({url: 'ajax.php',
            method: "post",
            data: { deconnexion:1 },
            dataType: 'json',
            el: '',
            fonction: function(el) {
              location.reload(true);
            }
    });
  });

  // Attribut title (infobulle) pour les icônes indiquant le niveau de protection
  $('.icon-lock1').attr('title','Visible uniquement par les utilisateurs connectés');
  $('.icon-lock2').attr('title','Visible uniquement par les élèves/colleurs/professeurs connectés');
  $('.icon-lock3').attr('title','Visible uniquement par les colleurs/professeurs connectés');
  $('.icon-lock4').attr('title','Visible uniquement par les professeurs connectés');
  $('.icon-lock5').attr('title','Complètement invisible (hors professeurs de cette matière)');

  // Menu pour les petits écrans
  $('.icon-menu').on("click", function() {
    $('#colonne,nav').toggleClass('visible');
  });

});
